#define WORK_MODE 0

#if (WORK_MODE==0)
#include "time_check.h"
#include <vector>
#include <numeric>

using namespace std;
int main()
{
	concurrency::array<int, 1> ar(100000);
	std::vector<int> v(100000);
	iota(begin(v), end(v), 0);

	TimeCheck tc(&ar.accelerator_view);
	tc.Start();
	concurrency::copy(v.data(), ar);
	double time = tc.GetTime();

	cout << "Happy Day! Time = " << time << " sec" << endl;
	return 0;
}
//	GPU  Ÿüũ
#elif (WORK_MODE==1)
#include <amp.h>
#include <iostream>
#include <vector>
#include <numeric>

using namespace std;
using namespace concurrency;

int main()
{
	//	const, discard_data example
	std::vector<int> in(5);
	iota(begin(in), end(in), 0);
	//	array_view Էθ 
	array_view<const int, 1> g_in(in.size(), in);
	//	array Էθ 
	const concurrency::array<int, 1> g_in2(in.size(), in.data());

	std::vector<int> out(5);
	concurrency::array_view<int, 1> g_out(in.size());	//	array_view
	concurrency::array<int, 1> g_out2(in.size());	//	array
	g_out.discard_data();
	//g_out2.discard_data();	//	array discard_data .

	concurrency::copy(g_in, g_out);
	concurrency::copy(g_out, out.data());	
	//	 : copy ͸ Է ϰ vectorԷ ȵȴ. copy(g_out, out(); (x)

	cout << "const, discard" << endl;
	std::for_each(begin(out), end(out), [&out](int v) {
		cout << v << endl;
	});

	//	section example
	std::vector<int> img(25);
	iota(begin(img), end(img), 0);
	array_view<const int, 2> g_img(5, 5, img);	//	5x5 image

	//	 0  1  2  3  4
	//	 5  6  7  8  9       
	//	10 11 12 13 14
	//	15 16 17 18 19
	//	20 21 22 23 24
	//	=> section
	//   6  7  8	
	//	11 12 13
	//	16 17 18

	array_view<const int, 2> g_partial = g_img.section(index<2>(1, 1), concurrency::extent<2>(3, 3));
	g_partial.synchronize();
	std::vector<int> out_small(9);
	concurrency::copy(g_partial, out_small.data());

	cout << "section" << endl;
	for (int i = 0; i < 9; i++)
	{
		cout << out_small[i] << " ";
		if ((((i + 1) % 3) == 0)) cout << endl;
	}
	return 0;
}
//	GPU  ּȭ, const, discard_data, section

#elif (WORK_MODE==2)
#include "time_check.h"
#include <vector>

using namespace std;
using namespace concurrency;

int main()
{
	std::vector<int> in(1000000, 100);
	concurrency::array<int, 1> g_in(in.size());

	TimeCheck tc(&g_in.accelerator_view);
	tc.Start();

	//	GPU 񵿱 縸 
	completion_future f = concurrency::copy_async(in.begin(), in.end(), g_in);
	//	other operation here
	cout << "GPU ߰    : ";
	Sleep(1000);	//	GPU ߰   Ǿ.

	f.get();


	double time = tc.GetTime();
	cout << time << endl;


	return 0;
}
//	GPU 񵿱⺹
#elif (WORK_MODE==3)
#include "time_check.h"
#include <vector>

using namespace std;
using namespace concurrency;

int main()
{
	accelerator acc = accelerator(accelerator::default_accelerator);
	if (acc.get_supports_cpu_shared_memory())
	{
		acc.set_default_cpu_access_type(access_type_read_write);
		accelerator_view av = acc.default_view;

		concurrency::extent<1> ex(5);	//	extent ݵ ־.
		concurrency::array<int, 1> a_mem_w(ex, av, access_type_write);
		concurrency::array<int, 1> a_mem_r(ex, av, access_type_read);
		concurrency::array<int, 1> a_mem_rw(ex, av, access_type_read_write);

		//	CPU 
		for (int i = 0; i < a_mem_rw.extent.size(); i++)
			a_mem_rw[i] = i;

		//	GPU ̿
		concurrency::parallel_for_each(a_mem_rw.extent, [&](index<1> index) restrict(amp)
		{
			a_mem_rw[index] = a_mem_rw[index] * 100;
		});

		//	CPU б
		for (int i = 0; i < a_mem_rw.extent.size(); i++)
			cout << a_mem_rw[i] << endl;

		return 0;
	}

	return 1;
}
//	 ޸ 
#elif (WORK_MODE==4)
#include "time_check.h"
#include <vector>
#include <numeric>

using namespace std;
using namespace concurrency;

int main()
{
	//	ӱ 
	accelerator_view gpu_av = accelerator(accelerator::default_accelerator).default_view;
	accelerator_view cpu_av = accelerator(accelerator::cpu_accelerator).default_view;

	//	¡ 
	concurrency::array<int, 1> dma_buf(100, cpu_av, gpu_av);
	array_view<int, 1> dma_view(dma_buf);
	dma_buf[0] = 100;	//	 dmaۿ ͸ ´. 
	cout << "DMA transfer 100 ? : " << dma_view(0) << endl;

	//	¡  
	std::vector<int> buf(10);
	iota(begin(buf), end(buf), 0);
	concurrency::array<int, 1> g_mem(buf.size(), buf.begin(), buf.end(), gpu_av);
	copy(g_mem, buf.data());
	std::for_each(begin(buf), end(buf), [](int v) { cout << v << " ";  });
	cout << endl;

	return 0;
}
//	¡ 迭
#endif
